@@ -1,7 +1,6 @@
-# coding=utf-8
 """A Python Package for parallelizing ssh connections via threading and multiprocessing.
 """
-# Copyright (C) 2015-2022 Jesse Almanrode
+# Copyright (C) 2015-2025 Jesse Almanrode
 #
 #     This program is free software: you can redistribute it and/or modify
 #     it under the terms of the GNU Lesser General Public License as published by
@@ -17,13 +16,14 @@
 #     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # For backwards compatibility
-from .ssh import SSH, envvars
-from .types import Command, EnvVars, Timeout, TimeoutTuple
-from .utils import ServerJob, Hook, sshread, echo, cpu_limit, shell_command
 import logging
 
+from .customtypes import Command, EnvVars, Timeout, TimeoutTuple
+from .ssh import SSH, envvars
+from .utils import Hook, ServerJob, cpu_limit, echo, shell_command, sshread
+
 __author__ = 'Jesse Almanrode (jesse@almanrode.com)'
-__all__ = ['ssh', 'types', 'utils']
+__all__ = ['ssh', 'customtypes', 'utils']
 
 log = logging.getLogger(__name__)
 log_handler = logging.StreamHandler()
