@@ -1,7 +1,6 @@
-# coding=utf-8
 """A wrapper for Paramiko that attempts to make ssh sessions easier to work with.
 """
-# Copyright (C) 2015-2022 Jesse Almanrode
+# Copyright (C) 2015-2025 Jesse Almanrode
 #
 #     This program is free software: you can redistribute it and/or modify
 #     it under the terms of the GNU Lesser General Public License as published by
@@ -17,12 +16,14 @@
 #     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import logging
 import os
-import paramiko
 import socket
 from getpass import getuser
 from typing import Any, Optional
 
-from .types import Command, EnvVars, Timeout
+import paramiko
+
+from .customtypes import Command, EnvVars, Timeout
+
 log = logging.getLogger('sshreader')
 
 
@@ -56,7 +57,7 @@ def envvars() -> EnvVars:
     return EnvVars(**env)
 
 
-class SSH(object):
+class SSH:
     """SSH Session class which can be used to send commands to a remote server.  It also supports basic SFTP commands
     and can be used to push/pull files from a remote system.
 
@@ -80,16 +81,14 @@ class SSH(object):
     """
     def __init__(self, fqdn: str, username: str, password: Optional[str] = None, keyfile: Optional[str] = None,
                  keypass: Optional[str] = None, port: int = 22, connect: bool = True, rsa_sha2: bool = True) -> None:
-        if not keyfile:
-            if len(paramiko.Agent().get_keys()) == 0:
-                if not all((username, password)):
-                    paramiko.SSHException('username and password or keyfile not provided')
+        if not keyfile and len(paramiko.Agent().get_keys()) == 0 and not all((username, password)):
+            paramiko.SSHException('username and password or keyfile not provided')
         self.host = fqdn
         self.username = username
         self.password = password
         if keyfile:
             if not isinstance(keyfile, str):
-                raise TypeError('expected %s for keyfile, got %s' % (str(str), str(type(keyfile))))
+                raise TypeError(f'expected {str(str)} for keyfile, got {str(type(keyfile))}')
             self.keyfile = os.path.abspath(os.path.expanduser(keyfile))
         else:
             self.keyfile = keyfile
@@ -122,7 +121,7 @@ class SSH(object):
         :return: Result of :meth:`paramiko.SFTPClient.put()`
         """
         if not self.__alive():
-            raise paramiko.SSHException("connection to %s not established" % (self.host,))
+            raise paramiko.SSHException(f"connection to {self.host} not established")
         sftp = paramiko.SFTPClient.from_transport(self._connection.get_transport())
         try:
             result = sftp.put(os.path.expanduser(srcfile), os.path.expanduser(dstfile))
@@ -140,7 +139,7 @@ class SSH(object):
         :return: None
         """
         if not self.__alive():
-            raise paramiko.SSHException("connection to %s not established" % (self.host,))
+            raise paramiko.SSHException(f"connection to {self.host} not established")
         sftp = paramiko.SFTPClient.from_transport(self._connection.get_transport())
         try:
             sftp.get(os.path.expanduser(srcfile), os.path.expanduser(dstfile))
@@ -164,7 +163,7 @@ class SSH(object):
         :raises: :class:`paramiko.SSHException`
         """
         if self.__alive() is False:
-            raise paramiko.SSHException("connection to %s not established" % (self.host, ))
+            raise paramiko.SSHException(f"connection to {self.host} not established")
         if combine:
             try:
                 stdin, stdout, stderr = self._connection.exec_command(command, timeout=timeout, get_pty=True)
@@ -226,12 +225,10 @@ class SSH(object):
         :raises: :class:`paramiko.SSHException`
         """
         if self.__alive():
-            raise paramiko.SSHException("connection to %s already established" % (self.host, ))
+            raise paramiko.SSHException(f"connection to {self.host} already established")
         paramiko.util.logging.getLogger().setLevel(logging.CRITICAL)  # Keeping paramiko from logging errors to stdout
-        if not self.keyfile:
-            if len(paramiko.Agent().get_keys()) == 0:
-                if not all((self.username, self.password)):
-                    paramiko.SSHException('username and password or keyfile not provided')
+        if not self.keyfile and len(paramiko.Agent().get_keys()) == 0 and not all((self.username, self.password)):
+            paramiko.SSHException('username and password or keyfile not provided')
         if self.keyfile:
             if self.rsa_sha2:
                 self._connection.connect(self.host, port=self.port, username=self.username, password=self.keypass,
